@@ -1,31 +1,37 @@
-/* apng2gif version 1.1
- * http://sourceforge.net/projects/apng2gif
+/* apng2gif version 1.3
  *
- * Copyright (c) 2010 Max Stepin
+ * This program converts APNG animations into animated GIF format.
+ * Wu quantizer is used for true-color files.
+ *
+ * http://apng2gif.sourceforge.net/
+ *
+ * Copyright (c) 2010-2011 Max Stepin
  * maxst at users.sourceforge.net
  *
- * GNU LGPL information
- * --------------------
+ * zlib license
+ * ------------
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * This software is provided 'as-is', without any express or implied
+ * warranty.  In no event will the authors be held liable for any damages
+ * arising from the use of this software.
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
+ * Permission is granted to anyone to use this software for any purpose,
+ * including commercial applications, and to alter it and redistribute it
+ * freely, subject to the following restrictions:
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * 1. The origin of this software must not be misrepresented; you must not
+ *    claim that you wrote the original software. If you use this software
+ *    in a product, an acknowledgment in the product documentation would be
+ *    appreciated but is not required.
+ * 2. Altered source versions must be plainly marked as such, and must not be
+ *    misrepresented as being the original software.
+ * 3. This notice may not be removed or altered from any source distribution.
  *
  */
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <zlib.h>
+#include "zlib.h"
 
 #define PNG_DISPOSE_OP_NONE        0x00
 #define PNG_DISPOSE_OP_BACKGROUND  0x01
@@ -41,7 +47,7 @@
     ((width) * (((unsigned int)(pixel_bits)) >> 3)) : \
     (( ((width) * ((unsigned int)(pixel_bits))) + 7) >> 3) )
 
-#define indx(r, g, b) ((r << 10) + (r << 6) + r + (g << 5) + g + b)
+#define indx(r, g, b) ((r << 12) + (r << 7) + r + (g << 6) + g + b)
 
 #define RED     2
 #define GREEN   1
@@ -60,18 +66,18 @@ unsigned char   trns[256];
 unsigned int    palsize, trnssize;
 unsigned short  trns1, trns2, trns3;
 int             trns_idx;
-int             tlevel, bcolor, back_r, back_g, back_b;
 unsigned int    accum;
 unsigned char   bits, codesize;
 unsigned char   buf[288];
 unsigned char   bigcube[128][128][128];
-int             wt[33*33*33]; 
-int             mr[33*33*33]; 
-int             mg[33*33*33];
-int             mb[33*33*33];
-double          m2[33*33*33];
-unsigned char   tag[33*33*33];
+int             wt[65*65*65]; 
+int             mr[65*65*65]; 
+int             mg[65*65*65];
+int             mb[65*65*65];
+double          m2[65*65*65];
+unsigned char   tag[65*65*65];
 typedef struct  {int r0; int r1; int g0; int g1; int b0; int b1; int vol; } box;
+int             tlevel, bcolor, back_r, back_g, back_b;
 
 int read32(unsigned int *val, FILE * f1)
 {
@@ -314,9 +320,10 @@ void compose3(unsigned char * dst1, unsigned int dstbytes1, unsigned char * dst2
 
     for (i=0; i<w; i++)
     {
+      col = sp[i];
+
       switch (depth)
       {
-        case 8: col = sp[i]; break;
         case 4: col = (sp[i>>1] & mask4[i&1]) >> shift4[i&1]; break;
         case 2: col = (sp[i>>2] & mask2[i&3]) >> shift2[i&3]; break;
         case 1: col = (sp[i>>3] & mask1[i&7]) >> shift1[i&7]; break;
@@ -342,7 +349,7 @@ void compose3(unsigned char * dst1, unsigned int dstbytes1, unsigned char * dst2
         else
         if (a != 0)
         {
-          if (a2 = (*dp2)>>24)
+          if ((a2 = (*dp2)>>24) != 0)
           {
             keep_original = 0;
             u = a*255;
@@ -407,7 +414,7 @@ void compose4(unsigned char * dst, unsigned int dstbytes, unsigned char * src, u
         else
         if (a != 0)
         {
-          if (a2 = (*dp)>>24)
+          if ((a2 = (*dp)>>24) != 0)
           {
             u = a*255;
             v = (255-a)*a2;
@@ -467,7 +474,7 @@ void compose6(unsigned char * dst, unsigned int dstbytes, unsigned char * src, u
         else
         if (a != 0)
         {
-          if (a2 = (*dp)>>24)
+          if ((a2 = (*dp)>>24) != 0)
           {
             u = a*255;
             v = (255-a)*a2;
@@ -491,7 +498,7 @@ void compose6(unsigned char * dst, unsigned int dstbytes, unsigned char * src, u
   }
 }
 
-int LoadAPNG(char * szImage, int *pWidth, int *pHeight, int *pColType, int *pFrames, int *pLoops, unsigned char **ppOut1, unsigned char **ppOut2, unsigned short ** ppDelays)
+int LoadAPNG(char * szIn, int *pWidth, int *pHeight, int *pColType, int *pFrames, int *pLoops, unsigned char **ppOut1, unsigned char **ppOut2, unsigned short ** ppDelays)
 {
   unsigned char     sig[8];
   unsigned int      i, j;
@@ -505,25 +512,25 @@ int LoadAPNG(char * szImage, int *pWidth, int *pHeight, int *pColType, int *pFra
   unsigned char     c, dop, bop;
   int               imagesize, zbuf_size, zsize;
   z_stream          zstream;
-  unsigned char *   pOut1;
-  unsigned char *   pOut2;
-  unsigned char *   pTemp;
-  unsigned char *   pData;
-  unsigned char *   pImg1;
-  unsigned char *   pImg2;
-  unsigned char *   pDst1;
-  unsigned char *   pDst2;
-  unsigned short *  pDelays;
-  FILE          *   f1;
+  unsigned char   * pOut1;
+  unsigned char   * pOut2;
+  unsigned char   * pTemp;
+  unsigned char   * pData;
+  unsigned char   * pImg1;
+  unsigned char   * pImg2;
+  unsigned char   * pDst1;
+  unsigned char   * pDst2;
+  unsigned short  * pDelays;
+  FILE            * f1;
   int               res = 1;
 
-  if ((f1 = fopen(szImage, "rb")) == 0)
+  if ((f1 = fopen(szIn, "rb")) == 0)
   {
-    printf("Error: can't open '%s'\n", szImage);
+    printf("Error: can't open '%s'\n", szIn);
     return res;
   }
 
-  printf("Reading '%s'...\n", szImage);
+  printf("Reading '%s'...\n", szIn);
 
   frames = 1;
   loops = 0;
@@ -533,17 +540,11 @@ int LoadAPNG(char * szImage, int *pWidth, int *pHeight, int *pColType, int *pFra
   x0 = 0;
   y0 = 0;
   bop = PNG_BLEND_OP_SOURCE;
-  palsize = 256;
+  palsize = 0;
   trnssize = 0;
   trns_idx = -1;
 
-  for (i=0; i<256; i++)
-  {
-    pal[i][0] = i;
-    pal[i][1] = i;
-    pal[i][2] = i;
-    trns[i] = 255;
-  }
+  memset(trns, 255, 256);
 
   zstream.zalloc = Z_NULL;
   zstream.zfree = Z_NULL;
@@ -552,7 +553,11 @@ int LoadAPNG(char * szImage, int *pWidth, int *pHeight, int *pColType, int *pFra
 
   do
   {
-    if (fread(sig, 1, 8, f1) != 8) break;
+    if (fread(sig, 1, 8, f1) != 8)
+    {
+      printf("Error: can't read the sig\n");
+      break;
+    }
     if (memcmp(sig, png_sign, 8) != 0) 
     {
       printf("Error: wrong PNG sig\n");
@@ -608,7 +613,7 @@ int LoadAPNG(char * szImage, int *pWidth, int *pHeight, int *pColType, int *pFra
     outimg1 = h*outrow1;
     outimg2 = h*outrow2;
 
-    pOut1   = (unsigned char  *)malloc((frames+1)*outimg1*2);
+    pOut1   = (unsigned char  *)malloc((frames+1)*outimg1*4);
     pOut2   = (unsigned char  *)malloc((frames+1)*outimg2);
     pDelays = (unsigned short *)malloc((frames+1)*2);
     pTemp   = (unsigned char  *)malloc(imagesize);
@@ -618,8 +623,8 @@ int LoadAPNG(char * szImage, int *pWidth, int *pHeight, int *pColType, int *pFra
       printf("Error: not enough memory\n");
       break;
     }
-    pImg1=pOut1;
-    pImg2=pOut2;
+    pImg1 = pOut1;
+    pImg2 = pOut2;
     memset(pOut1, 0, outimg1);
     memset(pOut2, 0, outimg2);
     pDelays[0] = 10;
@@ -654,7 +659,7 @@ int LoadAPNG(char * szImage, int *pWidth, int *pHeight, int *pColType, int *pFra
           {
             trns[i] = c;
             trnssize = i+1;
-            if (c == 0 && coltype == 3 && trns_idx == -1) 
+            if (c == 0 && coltype == 3 && trns_idx == -1)
               trns_idx = i;
           }
         }
@@ -704,7 +709,7 @@ int LoadAPNG(char * szImage, int *pWidth, int *pHeight, int *pColType, int *pFra
         free(pOut1);
         free(pOut2);
         free(pDelays);
-        pOut1   = (unsigned char  *)malloc((frames+1)*outimg1*2);
+        pOut1   = (unsigned char  *)malloc((frames+1)*outimg1*4);
         pOut2   = (unsigned char  *)malloc((frames+1)*outimg2);
         pDelays = (unsigned short *)malloc((frames+1)*2);
         pImg1   = pOut1;
@@ -719,10 +724,10 @@ int LoadAPNG(char * szImage, int *pWidth, int *pHeight, int *pColType, int *pFra
         {
           if (dop == PNG_DISPOSE_OP_PREVIOUS)
           {
-            if (coltype != 6)
+            if (coltype == 0 || coltype == 3)
               memcpy(pImg1 + outimg1, pImg1, outimg1);
-            if (coltype != 4)
-              memcpy(pImg2 + outimg2, pImg2, outimg2);
+
+            memcpy(pImg2 + outimg2, pImg2, outimg2);
           }
 
           pDst1 = pImg1 + y0*outrow1 + x0;
@@ -747,28 +752,25 @@ int LoadAPNG(char * szImage, int *pWidth, int *pHeight, int *pColType, int *pFra
 
             if (dop == PNG_DISPOSE_OP_BACKGROUND)
             {
-              pDst1 += outimg1;
-              pDst2 += outimg2;
+              if (coltype == 0 || coltype == 3)
+              {
+                if (trns_idx >= 0)
+                {
+                  pDst1 += outimg1;
+                  for (j=0; j<h0; j++)
+                  {
+                    memset(pDst1, trns_idx, w0);
+                    pDst1 += outrow1;
+                  }
+                }
+                else 
+                  keep_original = 0;
+              }
 
+              pDst2 += outimg2;
               for (j=0; j<h0; j++)
               {
                 memset(pDst2, 0, w0*4);
-                if (coltype == 0)
-                {
-                  if (trnssize) 
-                    memset(pDst1, trns[1], w0); 
-                  else 
-                    keep_original = 0;
-                }
-                else
-                if (coltype == 3)
-                {
-                  if (trns_idx >= 0) 
-                    memset(pDst1, trns_idx, w0); 
-                  else 
-                    keep_original = 0;
-                }
-                pDst1 += outrow1;
                 pDst2 += outrow2;
               }
             }
@@ -864,7 +866,7 @@ int LoadAPNG(char * szImage, int *pWidth, int *pHeight, int *pColType, int *pFra
         c = (unsigned char)(chunk & 0xFF);
         if (notabc(c)) break;
 
-        fseek( f1, len, SEEK_CUR );
+        fseek(f1, len, SEEK_CUR);
         if (read32(&crc, f1)) break;
       }
     }
@@ -892,31 +894,31 @@ void WuHistogram(int size, unsigned char *pOut1, unsigned char *pOut2)
 {
   int               i, j, r, g, b, inr, ing, inb;
   unsigned char  *  ps;
-  unsigned short *  pd;
+  unsigned int   *  pd;
   int               table[256];
         
   for (i=0; i<256; i++) 
     table[i]=i*i;
 
-  memset(wt, 0, 33*33*33*sizeof(int));
-  memset(mr, 0, 33*33*33*sizeof(int));
-  memset(mg, 0, 33*33*33*sizeof(int));
-  memset(mb, 0, 33*33*33*sizeof(int));
-  memset(m2, 0, 33*33*33*sizeof(double));
+  memset(wt, 0, 65*65*65*sizeof(int));
+  memset(mr, 0, 65*65*65*sizeof(int));
+  memset(mg, 0, 65*65*65*sizeof(int));
+  memset(mb, 0, 65*65*65*sizeof(int));
+  memset(m2, 0, 65*65*65*sizeof(double));
 
   ps = pOut2;
-  pd = (unsigned short *)pOut1;
+  pd = (unsigned int *)pOut1;
 
   for (i=0; i<size; i++)
   {
     r = *ps++;
     g = *ps++;
     b = *ps++;
-    if (*ps++ >= tlevel) 
+    if (*ps++ >= tlevel)
     {
-      inr = (r>>3)+1; 
-      ing = (g>>3)+1; 
-      inb = (b>>3)+1; 
+      inr = (r>>2)+1;
+      ing = (g>>2)+1;
+      inb = (b>>2)+1;
       *pd++ = j = indx(inr, ing, inb);
       wt[j]++;
       mr[j] += r;
@@ -931,27 +933,27 @@ void WuHistogram(int size, unsigned char *pOut1, unsigned char *pOut2)
 
 void WuMoments()
 {
-  unsigned short    ind1, ind2;
+  unsigned int      ind1, ind2;
   unsigned char     i, r, g, b;
   int               line, line_r, line_g, line_b;
   double            line2;
-  int               area[33], area_r[33], area_g[33], area_b[33];
-  double            area2[33];
+  int               area[65], area_r[65], area_g[65], area_b[65];
+  double            area2[65];
 
-  for (r=1; r<=32; r++)
+  for (r=1; r<=64; r++)
   {
-    for (i=0; i<=32; i++) 
+    for (i=0; i<=64; i++) 
     {
       area[i] = area_r[i] = area_g[i] = area_b[i] = 0;
       area2[i] = (double)0.0;
     }
 
-    for (g=1; g<=32; g++)
+    for (g=1; g<=64; g++)
     {
       line = line_r = line_g = line_b = 0;
       line2 = (double)0.0;
 
-      for (b=1; b<=32; b++)
+      for (b=1; b<=64; b++)
       {
         ind1 = indx(r, g, b);
         line += wt[ind1];
@@ -964,7 +966,7 @@ void WuMoments()
         area_g[b] += line_g;
         area_b[b] += line_b;
         area2[b] += line2;
-        ind2 = ind1 - 1089;
+        ind2 = ind1 - 65*65;
         wt[ind1] = wt[ind2] + area[b];
         mr[ind1] = mr[ind2] + area_r[b];
         mg[ind1] = mg[ind2] + area_g[b];
@@ -1165,22 +1167,20 @@ void ConvertTo8bit(int coltype, int size, unsigned char *pOut1, unsigned char *p
   unsigned char     r, g, b, a;
   unsigned char  *  ps;
   unsigned char  *  pd;
-  unsigned short *  pt;
-  unsigned char     bits[256];
+  unsigned int   *  pt;
+  unsigned int      bit[256];
   box               cube[256];
   double            temp;
   double            vv[256];
-    
+
+  memset(pal, 0, 256*3);
+
   if (coltype == 2 || coltype == 4 || coltype == 6 || keep_original == 0)
   {
     for (i=0; i<256; i++)
-      bits[i] = ((i>>7)&1) + ((i>>6)&1) + ((i>>5)&1) + ((i>>4)&1) + ((i>>3)&1) + ((i>>2)&1) + ((i>>1)&1) + (i&1);
+      bit[i] = ((i>>7)&1) + ((i>>6)&1) + ((i>>5)&1) + ((i>>4)&1) + ((i>>3)&1) + ((i>>2)&1) + ((i>>1)&1) + (i&1);
 
     memset(bigcube, 0, 128*128*128);
-
-    pal[0][0] = back_r;
-    pal[0][1] = back_g;
-    pal[0][2] = back_b;
     trans = 0;
     ps = pOut2;
 
@@ -1191,8 +1191,16 @@ void ConvertTo8bit(int coltype, int size, unsigned char *pOut1, unsigned char *p
         r = *ps++;
         g = *ps++;
         b = *ps++;
-        if (*ps++ >= tlevel) 
+        if (*ps++ >= tlevel)
           bigcube[r>>1][g>>1][b>>1] |= (1<<(((r&1)<<2) + ((g&1)<<1) + (b&1)));
+        else
+          trans = 1;
+      }
+      if (coltype == 2 && trnssize > 0)
+      {
+        pal[0][0] = (unsigned char)trns1;
+        pal[0][1] = (unsigned char)trns2;
+        pal[0][2] = (unsigned char)trns3;
       }
     }
     else
@@ -1214,16 +1222,22 @@ void ConvertTo8bit(int coltype, int size, unsigned char *pOut1, unsigned char *p
           }
           bigcube[r>>1][g>>1][b>>1] |= (1<<(((r&1)<<2) + ((g&1)<<1) + (b&1)));
         }
+        else
+          trans = 1;
       }
+      pal[0][0] = back_r;
+      pal[0][1] = back_g;
+      pal[0][2] = back_b;
     }
 
     colors8 = colors7 = trans;
+
     for (i=0; i<128; i++)
     for (j=0; j<128; j++)
     for (k=0; k<128; k++)
     if ((a = bigcube[i][j][k]) != 0)
     {
-      colors8 += bits[a];
+      colors8 += bit[a];
       colors7++;
     }
     printf("%d colors.\n", colors8);
@@ -1326,7 +1340,7 @@ void ConvertTo8bit(int coltype, int size, unsigned char *pOut1, unsigned char *p
       trns_idx = 0;
       n = 1;
       cube[1].r0 = cube[1].g0 = cube[1].b0 = 0;
-      cube[1].r1 = cube[1].g1 = cube[1].b1 = 32;
+      cube[1].r1 = cube[1].g1 = cube[1].b1 = 64;
 
       for (i=2; i<256; i++)
       {
@@ -1351,7 +1365,7 @@ void ConvertTo8bit(int coltype, int size, unsigned char *pOut1, unsigned char *p
           break;
       }
 
-      memset(tag, 0, 33*33*33);
+      memset(tag, 0, 65*65*65);
 
       for (c=1; c<palsize; c++)
       {
@@ -1374,12 +1388,134 @@ void ConvertTo8bit(int coltype, int size, unsigned char *pOut1, unsigned char *p
         }
       }
 
-      pt = (unsigned short *)pOut1;
+      pt = (unsigned int *)pOut1;
       pd = pOut1;
       for (i=0; i<size; i++)
         *pd++ = tag[*pt++];
     }
   }
+  else
+  if (coltype == 3)
+  {
+    if (bcolor == -1)
+    {
+      if (trns_idx == -1)
+      for (c=0; c<palsize; c++)
+      if (trns[c] < tlevel)
+      {
+        trns_idx = c;
+        break;
+      }
+
+      if (trns_idx >= 0)
+      {
+        ps = pOut1;
+        for (i=0; i<size; i++, ps++)
+        {
+          if (trns[*ps] < tlevel)
+            *ps = trns_idx;
+        }
+      }
+    }
+    else
+    {
+      for (c=0; c<palsize; c++)
+      {
+        if (trns[c] == 0)
+        {
+          pal[c][0] = back_r;
+          pal[c][1] = back_g;
+          pal[c][2] = back_b;
+        }
+        else
+        if (trns[c] < 255)
+        {
+          pal[c][0] = (pal[c][0]*trns[c] + back_r*(255-trns[c]))/255;
+          pal[c][1] = (pal[c][1]*trns[c] + back_g*(255-trns[c]))/255;
+          pal[c][2] = (pal[c][2]*trns[c] + back_b*(255-trns[c]))/255;
+        }
+      }
+    }
+  }
+  else
+  if (coltype == 0)
+  {
+    unsigned int    gray[256];
+
+    memset(gray, 0, 256*4);
+    palsize = 0;
+    ps = pOut1;
+
+    if (trns_idx == -1)
+      for (i=0; i<size; i++)
+        gray[*ps++] = 1;
+    else
+      for (i=0; i<size; i++, ps++)
+        if (*ps != trns_idx)
+          gray[*ps] = 1;
+
+    if (trns_idx == -1)
+    for (i=0; i<256; i++)
+    if (gray[i] == 0)
+    {
+      trns_idx = i;
+      break;
+    }
+
+    if (trns_idx >= 0)
+    {
+      if (bcolor != -1)
+      {
+        pal[0][0] = back_r;
+        pal[0][1] = back_g;
+        pal[0][2] = back_b;
+      }
+      else
+      if (trnssize > 0)
+      {
+        pal[0][0] = (unsigned char)trns1;
+        pal[0][1] = (unsigned char)trns1;
+        pal[0][2] = (unsigned char)trns1;
+      }
+      palsize++;
+    }
+
+    for (i=0; i<256; i++)
+    if (gray[i] != 0)
+    {
+      pal[palsize][0] = i;
+      pal[palsize][1] = i;
+      pal[palsize][2] = i;
+      palsize++;
+    }
+
+    ps = pOut1;
+
+    if (trns_idx == -1)
+    {
+      for (i=0; i<size; i++)
+      {
+        for (c=0; c<palsize; c++)
+          if (*ps==pal[c][0])
+            break;
+        *ps++ = c;
+      }
+    }
+    else
+    {
+      for (i=0; i<size; i++)
+      {
+        if (*ps == trns_idx)
+          c = 0;
+        else
+          for (c=1; c<palsize; c++)
+            if (*ps==pal[c][0])
+              break;
+        *ps++ = c;
+      }
+      trns_idx = 0;
+    }
+  }
 }
 
 int outcode(unsigned int code, FILE * f1)
@@ -1462,7 +1598,7 @@ int EncodeLZW(unsigned char * data, int datasize, FILE * f1, unsigned char minco
   return 0;
 }
 
-int SaveAGIF(char * szImage, unsigned char * pimg, unsigned short * pdelays, int w, int h, int frames, int loops)
+int SaveAGIF(char * szOut, unsigned char * pimg, unsigned short * pdelays, int w, int h, int frames, int loops)
 {
   unsigned char gif_head[13] = {0x47, 0x49, 0x46, 0x38, 0x39, 0x61, w&0xFF, w>>8, h&0xFF, h>>8, 0, (trns_idx >= 0) ? trns_idx : 0, 0};
   unsigned char netscape[19] = {0x21, 0xFF, 0x0B, 'N', 'E', 'T', 'S', 'C', 'A', 'P', 'E', '2', '.', '0', 3, 1, loops&0xFF, loops>>8, 0};
@@ -1481,9 +1617,9 @@ int SaveAGIF(char * szImage, unsigned char * pimg, unsigned short * pdelays, int
   int           res = 1;
   FILE        * f1;
 
-  if ((f1 = fopen(szImage, "wb")) == 0)
+  if ((f1 = fopen(szOut, "wb")) == 0)
   {
-    printf("Error: can't open '%s'\n", szImage);
+    printf("Error: can't open '%s'\n", szOut);
     return res;
   }
 
@@ -1493,7 +1629,7 @@ int SaveAGIF(char * szImage, unsigned char * pimg, unsigned short * pdelays, int
     return res;
   }
 
-  printf("Writing '%s'...\n", szImage);
+  printf("Writing '%s'...\n", szOut);
 
   do
   {
@@ -1513,10 +1649,10 @@ int SaveAGIF(char * szImage, unsigned char * pimg, unsigned short * pdelays, int
     if (fwrite(pal, 3, palsize, f1) != palsize) break;
     if (fwrite(netscape, 1, 19, f1) != 19) break;
 
-    x0 = 0;
-    y0 = 0;
-    w0 = w;
-    h0 = h;
+    x0 = x1 = 0;
+    y0 = y1 = 0;
+    w0 = w1 = w;
+    h0 = h1 = h;
 
     for (n=0; n<frames; n++)
     {
@@ -1664,9 +1800,11 @@ int main(int argc, char** argv)
   unsigned short * pDelays = NULL;
   int              i, w, h, coltype, frames, loops;
 
+  printf("\napng2gif 1.3\n\n");
+
   if (argc <= 1)
   {
-    printf("\nUsage : apng2gif anim.png [anim.gif] [/t tlevel] [/b bcolor]\n\n"
+    printf("Usage : apng2gif anim.png [anim.gif] [/t tlevel] [/b bcolor]\n\n"
            "tlevel: transparency threshold level (format: /t 128)\n"
            "bcolor: background blend color (format: /b #808080 or /b 128 128 128)\n"
            "When /b is used, /t is ignored. Default is /t 128, no bcolor.\n");
@@ -1714,7 +1852,7 @@ int main(int argc, char** argv)
           else
           if (szOpt[0] == '#')
           {
-            sscanf(szOpt+1, "%x", &bcolor);
+            sscanf(szOpt+1, "%x", (unsigned int*)(&bcolor));
             back_r = (bcolor>>16)&0xFF;
             back_g = (bcolor>>8)&0xFF;
             back_b = (bcolor)&0xFF;
@@ -1725,6 +1863,12 @@ int main(int argc, char** argv)
             back_r = atoi(argv[i]);
             back_g = atoi(argv[++i]);
             back_b = atoi(argv[++i]);
+            if (back_r < 0)   back_r = 0;
+            if (back_r > 255) back_r = 255;
+            if (back_g < 0)   back_g = 0;
+            if (back_g > 255) back_g = 255;
+            if (back_b < 0)   back_b = 0;
+            if (back_b > 255) back_b = 255;
             bcolor = (back_r<<16) + (back_g<<8) + back_b;
           }
         }
@@ -1741,7 +1885,7 @@ int main(int argc, char** argv)
   if (szOut[0] == 0)
   {
     strcpy(szOut, szIn);
-    if (szExt = strrchr(szOut, '.')) *szExt = 0;
+    if ((szExt = strrchr(szOut, '.')) != NULL) *szExt = 0;
     strcat(szOut, ".gif");
   }
 
@@ -1750,6 +1894,7 @@ int main(int argc, char** argv)
     printf("Error: can't load '%s'\n", szIn);
     return 1;
   }
+  printf("%d frame%s.\n", frames, (frames==1) ? "" : "s");
 
   ConvertTo8bit(coltype, w*h*frames, pOut1, pOut2);
 
